@@ -12,8 +12,8 @@ from . import solve
 from . import utils
 
 from .region import Region
-from .forms import IntegralForm, IntegralFormMixed
-from .field import Field
+from .forms import IntegralForm, IntegralFormMixed, IntegralFormAxisymmetric
+from .field import Field, FieldAxisymmetric
 from .doftools import Boundary
 
 
